Package: Morgan2013
Type: Package
Title: Supporting material, CSAMA2013
Version: 1.0.1
Author: Marc Carlson, Valerie Obenchain, Herv\'e Pag\`es, Paul
    Shannon, Dan Tenenbaum, Martin Morgan
Maintainer: Biocore Team c/o BioC user list
        <bioconductor@stat.math.ethz.ch>
License: Artistic-2.0
Description: This package contains supporting material for lectures
    and labs delivered by Martin Morgan during the CSAMA 2013 course.
Imports: BiocInstaller
Suggests: Biostrings, ShortRead, Rsamtools, GenomicRanges, rtracklayer,
        RNAseqData.HNRNPC.bam.chr14, org.Hs.eg.db,
        BSgenome.Hsapiens.UCSC.hg19, TxDb.Hsapiens.UCSC.hg19.knownGene,
        Homo.sapiens, TxDb.Dmelanogaster.UCSC.dm3.ensGene, GO.db,
        MotifDb, seqnames.db, SNPlocs.Hsapiens.dbSNP.20120608,
        GenomicFeatures, biomaRt, AnnotationHub, VariantAnnotation,
        ensemblVEP, VariantTools, gmapR, LungCancerLines, Gviz, shiny,
        parallel, lattice, edgeR
Packaged: 2013-07-05 22:30:37 UTC; mtmorgan
