library(shiny)
library(parathyroidSE)

shinyServer(function(input, output) {

    dataset <- reactive({
        data(parathyroidGenesSE)
        parathyroidGenesSE
    })

    output$overview <- renderPrint({
        print(dataset())
    })

})
