library(shiny)

shinyUI(pageWithSidebar(

  headerPanel("SummarizedExperiment Viewer"),

  sidebarPanel(
    checkboxInput("show_expt_data",
                  "Experiment data"),
    checkboxInput("show_col_data",
                  "Column data"),
    selectInput("transform", "Transform",
        choices = c("none", "asinh", "sqrt",
          "log1"))
    ),
   
  mainPanel(
      verbatimTextOutput("overview"))

))
