%% \VignetteIndexEntry{Lab: Annotation, Variants, and Visualization}

\documentclass[a4paper]{report}

\usepackage{BioconductorVignette}
\graphicspath{{annotation-visualization-variants/}}

\title{Annotations, Visualization, and Variants}
\author{
  Marc Carlson, Valerie Obenchain, Herv\'e Pag\`es, Paul Shannon,
  Dan Tenenbaum, Martin
  Morgan\footnote{\href{mailto:mtmorgan@fhcrc.org}{mtmorgan@fhcrc.org}}
}
\date{June 23 -- 28, 2013}

<<setup, echo=FALSE>>=
library(Morgan2013)
@ 

\begin{document}

\maketitle
\tableofcontents

\part{Annotation and Visualization (Intermediate)}

\include{annotation-visualization-variants/D1_GeneAnnotations}
\include{annotation-visualization-variants/D2_GenomicAnnotations}
\include{annotation-visualization-variants/D3_Visualization}

\part{Variants (Advanced)}

\include{annotation-visualization-variants/D4_CalledVariants}
\include{annotation-visualization-variants/D5_StructuralVariants}
\include{annotation-visualization-variants/D6_RegulatoryVariants}

\addcontentsline{toc}{chapter}{References}
\bibliographystyle{abbrv}
\bibliography{References}

\end{document}
