%% \VignetteIndexEntry{Lab: Sequences, Reads, Alignments and Ranges}

\documentclass[a4paper]{report}

\usepackage{BioconductorVignette}
\graphicspath{{sequences-reads-ranges-alignments/}}

\title{Sequences, Reads, Alignments, and Ranges}
\author{
  Marc Carlson, Valerie Obenchain, Herv\'e Pag\`es, Paul Shannon,
  Dan Tenenbaum, Martin
  Morgan\footnote{\href{mailto:mtmorgan@fhcrc.org}{mtmorgan@fhcrc.org}}
}
\date{June 23 -- 28, 2013}

<<setup, echo=FALSE>>=
library(Morgan2013)
@ 

\begin{document}

\maketitle
\tableofcontents

\include{sequences-reads-ranges-alignments/A1_SequenceAnalysis}
\include{sequences-reads-ranges-alignments/A2_ReadsAndStrings}
\include{sequences-reads-ranges-alignments/A3_AlignmentsAndRanges}
\include{sequences-reads-ranges-alignments/A4_LargeData}

\addcontentsline{toc}{chapter}{References}
\bibliographystyle{abbrv}
\bibliography{References}

\end{document}
