\name{IntermediateSequenceAnalysis2013-package}
\Rdversion{1.1}
\alias{IntermediateSequenceAnalysis2013-package}
\alias{IntermediateSequenceAnalysis2013}
\docType{package}

\title{Supporting material, CSAMA 2013}

\description{

  This package contains supporting material for lectures and labs
  delivered by Martin Morgan during the CSAMA 2013 course.

}

\author{

  Author: Marc Carlson, Valerie Obenchain, Herv\'e Pag\`es, Paul
  Shannon, Dan Tenenbaum, Martin Morgan

  Maintainer: Biocore Team c/o BioC user list
  <bioconductor@stat.math.ethz.ch>

}

\keyword{ package }

\examples{
packageDescription("Morgan2013")
}
